package processModel;

import java.awt.Image;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.JApplet;
import javax.swing.JComponent;

import processView.Externaliser;
import processView.ImagePanel;
import processView.ProcessApplet;
import utilities.AppConstants;

public class Formula
{
	private String group;
	private String name;
	private String expression;
	private String[] equationElements;
	private boolean mustRearrange;
	private String imageFile;
	private List<Formula> rearrangedEquations;
	private Formula parentEquation;
	private int unknownID;
	private URL base;
	
	public Formula (String group, String name, String[] equationElements, boolean requiresEvaluation, boolean mustRearrange, URL base, String imageFile){
		this.group = group;
		this.name = name;
		this.equationElements = equationElements;
		this.mustRearrange = mustRearrange;
		this.base = base;
		
		this.imageFile = imageFile;
		rearrangedEquations = new ArrayList<Formula>();
		
		String baseString = base.toString();
		try {
			this.base = new URL(baseString.substring(0, baseString.lastIndexOf("/")));
		}
		catch (MalformedURLException ex) {
			ex.printStackTrace();
		}
	}
	
	public String formulaForASPIRE()
	{
		String equation = "";
		for (String element : equationElements) {
			equation += element;
		}
		equation = equation.replaceAll(" ", "");
		return equation;
	}
	
	public String name (){ return name; }
	public String imageFile () { return imageFile; }
	public String[] equationElements() { return equationElements; }
	public boolean mustRearrange() { return mustRearrange; }
	
	/**
	 * Creates and returns a JComponent containing the equation image
	 * @param applet JApplet which will contain/display this equation
	 * @return JComponent The component containing the equation image
	 */
	public JComponent imageComponent (ProcessApplet applet){
		Image imgEquation = applet.getImageCache().getImage(imageFile);
		
		JComponent imagePanel = new ImagePanel(imgEquation);
		return imagePanel;
	}
	
	public String toString()
	{
		return name;
	}
	
	public void setParentEquation(Formula parentEquation)
	{
		this.parentEquation = parentEquation;
	}
	
	public boolean isRearrangedEquation()
	{
		return parentEquation != null;
	}
	
	public void addRearrangedEquation(Formula rearrangedEquation)
	{
		rearrangedEquations.add(rearrangedEquation);
	}
	
	public boolean canRearrange()
	{
		return rearrangedEquations.size() > 0;
	}
	
	public java.util.List<Formula> getRearrangedEquations()
	{
		Collections.shuffle(rearrangedEquations);
		return rearrangedEquations;
	}
	
	public void setExpression(String expression) {
		this.expression = expression.replaceAll(" ", "");
	}
	public void setUnknownID(int unknownID) { this.unknownID = unknownID; }
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		
		// TODO:  Shouldn't really do getNextID() here, instead this should be done at construction time
		xmlStr += Externaliser.openInstanceTag(indent, Process.getNextID(), group);
		xmlStr += formulaUnknownASPIRE_XML(indent);
		xmlStr += Externaliser.openRelatedSlotTag(indent, 5);
		xmlStr += Externaliser.relatedInstanceTag(indent + AppConstants.Sx4, unknownID);
		xmlStr += Externaliser.closeRelatedSlotTag(indent);
		xmlStr += Externaliser.closeInstanceTag(indent);							

		return xmlStr;
    }
	
	protected String formulaUnknownASPIRE_XML(String indent) {
		String xmlStr = "";
		
		if (parentEquation != null) {
			// Name
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 1, parentEquation.name);
			// Original Formula
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 2, parentEquation.formulaForASPIRE());
			// Rearranged Formula
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 3, formulaForASPIRE());
		}
		else {
			// Name
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 1, name);
			// Original Formula
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 2, formulaForASPIRE());
			// Rearranged Formula
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 3, "");
		}
		
		// Expression
		if (expression != null) {
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 4, expression);
		}
		else {
			xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 4, "");
		}
		return xmlStr;
	}
	
}
