package processModel;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Dimension2D;
import java.util.Observable;

import utilities.AppConstants;
import utilities.HandleType;

public abstract class Handle extends Observable implements Moveable{


	private int handleSize;
	private boolean connected;
	private boolean wasConnected;
	private Element parent;
	private Point origin;
	private boolean selected = false;
	private HandleType handleType;
	protected final int MASK_INCREMENT = 5;
	private int cursorType = Cursor.CROSSHAIR_CURSOR;

	private Handle() {
		super();
	}
	
	public Handle(Point origin, HandleType handleType){
		this.handleType(handleType);
		this.origin(origin);
		this.handleSize(AppConstants.MINIMUM_HANDLE_SIZE);
	}

	public Handle pointIsInHandle (Point p){
		Point topLeftMaskPoint = new Point (((int) origin().getX() - (handleSize()/2) - MASK_INCREMENT),((int) origin().getY( )- (handleSize()/2) - MASK_INCREMENT));
		Dimension2D rectMaskDimension = new Dimension (handleSize() + (MASK_INCREMENT * 2), handleSize() +  (MASK_INCREMENT * 2));
		
		Rectangle rectMask = new Rectangle();
		rectMask.setFrame(topLeftMaskPoint, rectMaskDimension);		
		
		if (rectMask.contains(p)){
			selected = true;
			return this;
		}
		else { 
			selected = false;
			return null;
		}
	}
	
	public void move(Point newPoint) {
		origin = newPoint;
		this.setChanged();
		notifyObservers(AppConstants.MOVE_ELEMENT);
	}
	
	public boolean moveHandleResizingOwner(Point newPoint){
		origin = newPoint;
		this.setChanged();
		notifyObservers(AppConstants.RESIZE_OWNER);
		return true;
	}

	public boolean selected() {
		return selected;
	}

	public void select(boolean select) {
		this.selected = select;
	}
	
	protected void origin(Point origin){
		this.origin = origin;
	}
	
	public Point origin(){
		return origin;
	}
	
	protected HandleType handleType() {
		return handleType;
	}
	
	protected void handleType (HandleType handleType){
		this.handleType = handleType;
	}

	protected void handleSize(int size) {
		if (size >= AppConstants.MINIMUM_HANDLE_SIZE)
			handleSize = size;
		else
			handleSize = AppConstants.MINIMUM_HANDLE_SIZE;
	}

	protected int handleSize (){
		return handleSize;
	}
	
	public Cursor getCursorType(){
		return Cursor.getPredefinedCursor(this.cursorType);
	}
	
	protected boolean setCursorType(int cursorType){
		boolean setWasSuccessful = false;
		
		try{
			Cursor cursor = Cursor.getPredefinedCursor(cursorType);
			if (cursor != null){
				this.cursorType = cursorType;
				setWasSuccessful= true;
			}
		} catch (IllegalArgumentException e){
			// Do nothing - use the default cursor, and return false
		}
		return setWasSuccessful;
	}
	
	protected abstract Color getHandleColor();

	public abstract void paint(Graphics2D g2d);
	
	public void prepareForDeletion(){
		this.deleteObservers();
	}
	
	public boolean isConnected()
	{
		return connected;
	}
	
	public boolean wasConnected()
	{
		return wasConnected;
	}
	
	public void setConnected(boolean connected)
	{
		this.connected=connected;
	}
	
	public void setWasConnected()
	{
		wasConnected=true;
	}
	
	public Element getParent() {
		return parent;
	}
}