package processView;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import processModel.Formula;
import processModel.State;
import processModel.Transition;
import processModel.Unknown;
import utilities.AppConstants;

public class FormulaWorkspace extends JPanel {

	private ProcessApplet controller;
	private JScrollPane scrollPane;
	private JPanel workspacePanel;
	private ArrayList<FormulaPanel> formulaPanels;
	private JButton btnAdd;
	private String lastStep;
	
	public FormulaWorkspace(ProcessApplet controller) {
		this.controller = controller;
		formulaPanels = new ArrayList<FormulaPanel>();
		
		initGUI();
	}
	
	private void initGUI() {
		workspacePanel = new JPanel();
		workspacePanel.setLayout(new BoxLayout(workspacePanel, BoxLayout.Y_AXIS));		
		
		scrollPane = new JScrollPane(workspacePanel);		
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setPreferredSize(new Dimension(320, FormulaPanel.PANEL_HEIGHT + 10));
				
		this.setLayout(new BorderLayout());
		this.add(scrollPane, BorderLayout.CENTER);
		
		// Add default formula panel
		//addFormulaPanel();
		
		JPanel controlBar = new JPanel();
		
		btnAdd = new JButton("Calculate another unknown");
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed (ActionEvent e) {
				addFormulaPanel();
			}
		});
		controlBar.add(btnAdd);		
		controlBar.add(Box.createHorizontalStrut(20));
				
		this.add(controlBar, BorderLayout.SOUTH);
				
	}
	
	public FormulaPanel getCurrentFormulaPanel()
	{
		return formulaPanels.get(formulaPanels.size()-1);
	}
	
	private List<Unknown> getUnknowns()
	{
		List<Unknown> unknowns = new ArrayList<Unknown>();
		for (State state : controller.getProcess().getStates())
		{
			unknowns.addAll(state.getUnknowns());
		}
		for (Transition transition : controller.getProcess().getTransitions())
		{
			unknowns.addAll(transition.getUnknowns());
		}
		return unknowns;
	}
	
	private List<Unknown> getCalculatedValues()
	{
		List<Unknown> unknowns = new ArrayList<Unknown>();
		for (State state : controller.getProcess().getStates())
		{
			System.out.println("Getting unknowns for state: " + state.getID());
			unknowns.addAll(state.getCalculatedValues());
		}
		for (Transition transition : controller.getProcess().getTransitions())
		{
			System.out.println("Getting unknowns for transition: " + transition.getID());
			unknowns.addAll(transition.getCalculatedValues());
		}
		
		// TODO:  Hack alert
		unknowns.add(getFinalPanel().getSelectedUnknown());
		
		return unknowns;
	}
	
	private List<Formula> getFormulas()
	{
		List<Formula> formulas = new ArrayList<Formula>();
		for (FormulaPanel panel : formulaPanels) {
			Formula formula = panel.getSelectedFormula();
			if (formula != null) {
				formulas.add(formula);
			}
		}
		return formulas;
	}
	
	
	public State getState(int id)
	{
		for (State state : controller.getProcess().getStates())
		{
			if (state.getID() == id)
			{
				return state;
			}
		}
		return null;
	}
	
	public void setLastStep(String step) {
		this.lastStep = step;
	}
	
	/*private void calculationComplete(FormulaPanel panel) {
		FormulaPanel finalPanel = getFinalPanel();
		controller.checkFormulaStep(panel, toASPIRE_XML(""+formulaPanels.size(), UNKNOWN_VALUE, 
				finalPanel.getSolution(UNKNOWN_VALUE)), "UnknownValue", finalPanel.getSolution(UNKNOWN_VALUE));		
		
		//JOptionPane.showMessageDialog(null, "Well done!, almost there. You have 100 mistakes in your solution");
	}*/
	
	public void addFormulaPanel() {
		FormulaPanel newPanel = new FormulaPanel(this);
		this.formulaPanels.add(newPanel);
		newPanel.setUnknowns(getUnknowns());
		workspacePanel.add(newPanel);
		this.validate();
		if (scrollPane.getViewport().getHeight() < workspacePanel.getHeight()) {
			Point p = scrollPane.getViewport().getViewPosition();
			p.y = workspacePanel.getHeight();
			scrollPane.getViewport().setViewPosition(p);
		}
		btnAdd.setEnabled(false);
	}
	
	public void removePanel(FormulaPanel panel)
	{
		formulaPanels.remove(panel);
		workspacePanel.remove(panel);
	}
	
	public void enableAddButton()
	{
		btnAdd.setEnabled(true);
	}
	
	private FormulaPanel getFinalPanel() {
		return formulaPanels.get(formulaPanels.size()-1);
	}
	
	
	public void setEnabled(boolean enabled) {
		super.setEnabled(enabled);
		btnAdd.setEnabled(enabled);	
	}
	
	public void evaluateCurrentEquation()
	{
		if (formulaPanels != null && formulaPanels.size() > 0)
			formulaPanels.get(formulaPanels.size()-1).evaluateEquation(false);
	}
	
	public static String UNKNOWN = "Unknown";
	public static String FORMULA = "Formula";
	public static String REARRANGED_FORMULA = "RearrangedFormula";
	public static String UNKNOWN_VALUE = "UnknownValue";
	
	/*public void enableNextStep() {
		if (lastStep.equals(UNKNOWN)) {
			getFinalPanel().setComponentEnabled(FORMULA);
		} else if (lastStep.equals(FORMULA)) {
			getFinalPanel().setComponentEnabled(REARRANGED_FORMULA);
		} else {
			getFinalPanel().setComponentEnabled(UNKNOWN_VALUE);
		}
	
	}*/
	
	public String getStepSuccessMessage() {
		if (lastStep.equals(UNKNOWN)) {
			return "<p>Now choose the correct formula for calculating the chosen unknown.</p>";
		} else if (lastStep.equals(FORMULA)) {
			return "<p>You are correct, you can rearrange the formula to make the unknown the subject.</p>";  
		} else if (lastStep.equals(REARRANGED_FORMULA)){
			return "<p>Great, now solve the equation and calculate the value of the unknown.</p>";
		} else {
			return "<p>Well done, you have correctly calculated the value of the unknown.</p>"
			 	+ "<p>Now see whether there are any other unknowns to be calculated. If you have completed calculating all unkowns then click on the 'Done' button.</p>";
		}
	}
	
	
	public void checkAnswer(FormulaPanel panel, String step, String value) {
		this.lastStep = step;
		controller.checkFormulaStep(panel, toASPIRE_XML(""+formulaPanels.size(), step, value), step, value);		
	}
		
	public String solvingUnknownsToASPIREXML() {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		System.out.println("Going to externalizer");
		getExternaliser().saveUnknownsAndFormulasToASPIREXML(new PrintWriter(bos), controller.getUserName(), controller.getOntologyID(), "Formula", getCalculatedValues(), getFormulas());
		return bos.toString();
	}
	
	/*public String selectingFormulaToASPIREXML() {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		getExternaliser().saveUnknownsAndFormulasToASPIREXML(new PrintWriter(bos), controller.getUserName(), controller.getOntologyID(), "FormulaSelection", getCalculatedValues(), getFormulas());
		return bos.toString();
	}
	
	public String rearrangingFormulaToASPIREXML() {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		getExternaliser().saveUnknownsAndFormulasToASPIREXML(new PrintWriter(bos), controller.getUserName(), controller.getOntologyID(), "FormulaRearrangement", getCalculatedValues(), getFormulas());
		return bos.toString();
	}*/
	
	private Externaliser getExternaliser()
	{
		return controller.getProcess().getExternaliser();
	}
	
	public ProcessApplet getController()
	{
		return controller;
	}
	
	
	public String toASPIRE_XML(String stepIteration, String step, String solution) {
		String xmlStr = "";
		
		xmlStr += Externaliser.openTag(AppConstants.S, Externaliser.ELT_PROBLEM_SOLVING_STEP 
					+ Externaliser.attribValuePair(AppConstants.ATT_ID, "1")
					+ Externaliser.attribValuePair(AppConstants.ATT_REPEATABLE, "true"));
		
		xmlStr += Externaliser.openTag(AppConstants.S, Externaliser.ELT_PROBLEM_SOLVING_STEP_ITERATION 
				+ Externaliser.attribValuePair(AppConstants.ATT_ID, stepIteration));
		
    	//Unknown
		String status;
		
		if (step.equals(UNKNOWN)) {
			status = AppConstants.AVAILABLE;
		} else {
			status = AppConstants.NOT_AVAILABLE;
		}		
    	xmlStr += Externaliser.openTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT 
    				+ Externaliser.attribValuePair(AppConstants.ATT_LABEL, UNKNOWN)
    				+ Externaliser.attribValuePair(AppConstants.ATT_STATUS, status));
    	if (step.equals(UNKNOWN)) {
    		xmlStr +=  Externaliser.openInstanceTag(AppConstants.Sx4, 10001+formulaPanels.size(),
    				"Unknown");
    		xmlStr +=  Externaliser.valueSlotTag(AppConstants.Sx6, 0, solution);
    		xmlStr +=  Externaliser.closeInstanceTag(AppConstants.Sx4);
    	}
    	xmlStr += Externaliser.closeTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT);
    	
    	// Formula
    	if (step.equals(FORMULA)) {
    		status = AppConstants.AVAILABLE;
		} else {
			status = AppConstants.NOT_AVAILABLE;
		}	
    	xmlStr += Externaliser.openTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT 
				+ Externaliser.attribValuePair(AppConstants.ATT_LABEL, FORMULA)
				+ Externaliser.attribValuePair(AppConstants.ATT_STATUS, status));
    	if (step.equals(FORMULA)) {
    		xmlStr +=  Externaliser.openInstanceTag(AppConstants.Sx4, 10002+formulaPanels.size(),
    				solution);
    		xmlStr +=  Externaliser.closeInstanceTag(AppConstants.Sx4);
    	}

    	xmlStr += Externaliser.closeTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT);
    	
    	
    	// Substituted formula
    	if (step.equals(REARRANGED_FORMULA)) {
    		status = AppConstants.AVAILABLE;
		} else {
			status = AppConstants.NOT_AVAILABLE;
		}	
    	xmlStr += Externaliser.openTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT 
				+ Externaliser.attribValuePair(AppConstants.ATT_LABEL, REARRANGED_FORMULA)
				+ Externaliser.attribValuePair(AppConstants.ATT_STATUS, status));
    	if (step.equals(REARRANGED_FORMULA)) {
    		xmlStr +=  Externaliser.openInstanceTag(AppConstants.Sx4, 10003+formulaPanels.size(), 
    				"Rearranged Formula");
    		xmlStr +=  Externaliser.valueSlotTag(AppConstants.Sx6, 0, solution);
    		xmlStr +=  Externaliser.closeInstanceTag(AppConstants.Sx4);
    	}
    	xmlStr += Externaliser.closeTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT);
    	
    	// UnknownValue
    	if (step.equals(UNKNOWN_VALUE)) {
    		status = AppConstants.AVAILABLE;
		} else {
			status = AppConstants.NOT_AVAILABLE;
		}	
    	xmlStr += Externaliser.openTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT 
				+ Externaliser.attribValuePair(AppConstants.ATT_LABEL, UNKNOWN_VALUE)
				+ Externaliser.attribValuePair(AppConstants.ATT_STATUS, status));
    	if (step.equals(UNKNOWN_VALUE)) {
    		xmlStr +=  Externaliser.openInstanceTag(AppConstants.Sx4, 10004+formulaPanels.size(), 
    				"Solution");
    		xmlStr +=  Externaliser.valueSlotTag(AppConstants.Sx6, 0, solution);
    		xmlStr +=  Externaliser.closeInstanceTag(AppConstants.Sx4);
    	}
    	xmlStr += Externaliser.closeTag(AppConstants.Sx2, Externaliser.ELT_SOLUTION_COMPONENT);
    	    	
    	
    	xmlStr += Externaliser.closeTag(AppConstants.S, Externaliser.ELT_PROBLEM_SOLVING_STEP_ITERATION);
    	xmlStr += Externaliser.closeTag(AppConstants.S, Externaliser.ELT_PROBLEM_SOLVING_STEP); 
		
		return xmlStr;

	}
}

