package processView;

import java.awt.*;
import javax.swing.*;

/**
 * An ImagePanel JComponent object 
 * displaying a given image
 * 
 * @author Moffat Mathews
 * @version 0.1 20 February 2006
 */


public class ImagePanel extends JComponent {
	/**
	 * 
	 */
	private static final long serialVersionUID = -2319350724971295090L;
	private Image theImage;

	 /**
	  * Constructor
	  * @param image The <code>Image</code> object
	  */
	public ImagePanel(Image image) {
		setImage(image);
	}

	/**
	 * Accessor that returns the <code>Image</code> object
	 * @return Image The equation image
	 */
	public Image getImage() {
		return theImage;
	}

	/**
	 * Setter that sets the <code>Image</code> object to be displayed.
	 * The image width and height are 
	 * calculated and the preferred size set
	 * @param image
	 */
	public void setImage(Image image) {
		if (image == null) {
			this.theImage = null;
			return;
		}

		int width = image.getWidth(this);

		if (width <= 0) {
			MediaTracker tracker = new MediaTracker(this);
			tracker.addImage(image, 0);

			try {
				tracker.waitForID(0);
			}
			catch (InterruptedException ie) {}

			width = image.getWidth(this);
		}

		int height = image.getHeight(this);

		this.setPreferredSize(new Dimension(width, height));
		this.theImage = image;
	}

	/**
	 * Overiden method that paints the image onto the panel
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.drawImage(theImage, 0, 0, this);
	}
	/**
	* This method should return an instance of this class which does 
	* NOT initialize it's GUI elements. This method is ONLY required by
	* Jigloo if the superclass of this class is abstract or non-public. It 
	* is not needed in any other situation.
	 */
	public static Object getGUIBuilderInstance() {
		return new ImagePanel(Boolean.FALSE);
	}
	
	/**
	 * This constructor is used by the getGUIBuilderInstance method to
	 * provide an instance of this class which has not had it's GUI elements
	 * initialized (ie, initGUI is not called in this constructor).
	 */
	public ImagePanel(Boolean initGUI) {
		super();
	}
}
